from cocos.layer import Layer
from cocos.scene import Scene
from cocos.tiles import MapLayer
from cocos.tiles import RectCell

import logging
log = logging.getLogger('Board')

#
# Local imports
#
from cell import *
from shape import *


class Board(Layer):
    is_event_handler = False
    def __init__(self, w, h, premade = None):
        super(Board,self).__init__()
        
        self.cells = {}
        
        self.boardWidth = w
        self.boardHeight = h
        
        self.SetupBoard()
        if (premade != None):
            self.SetupPremade(premade)
        
    def SetupBoard(self):
        self.cells = {}
        for x in range( self.boardWidth ):
            for y in range( self.boardHeight ):
                self.cells[ (x,y) ] = None
        

        log.info("Cells: "+str(self.boardWidth)+":"+str(self.boardHeight))

                
    def SetupPremade(self, premade):
        
        for x in range( self.boardWidth ):
            for y in range( self.boardHeight ):
                if (premade[x][y] != 0):
                    cell =  MakeCell(premade[x][y]) 
                    cell.ConvertCellPosToPosition(x, y)
                    self.add(cell)
                    self.cells[ (x,y) ] = cell
                else:
                    self.cells[ (x,y) ] = None
        
            
    def ApplyGravity(self, x, y):
        y -= 1
        while y > 0 :#self.boardHeight-1:
            #log.info("ApplyGravity checking "+str((x,y)) + " till "+str(self.boardHeight))
            if (self.cells[ (x,y+1) ] == None and self.cells[ (x,y) ] != None):
                #log.info("Applying gravity to "+str((x,y)))
                self.cells[ (x,y+1) ] = self.cells[ (x,y) ]
                
                # Ugly
                
                self.cells[ (x,y+1) ].x = int(100+x*32)
                self.cells[ (x,y+1) ].y = 600-int(100+(y+1)*32)

                self.cells[ (x,y) ] = None
            y -= 1
    
    def GetColumn(self, col):
        if (col <0 or col >= self.boardWidth):
            return None
        
        return self.cells[col]    
    
    def GetCell(self, pos):
        if (pos[0] <0 or pos[0] >= self.boardWidth or pos[1] <0 or pos[1]>=self.boardHeight):
            return None
        
        return self.cells[pos]
    
    def GetCellID(self, pos):
        
        cell = self.GetCell(pos)
        if (cell == None):
            return 0
        
        return cell.cell_ID
    
    def SetCell(self, pos, value):
        self.cells[pos] = value
        
    def Clear(self):
        for x in range( self.boardWidth ):
            for y in range( self.boardHeight ):
                 self.cells[ (x,y) ] = None
                    