import math
import random
import string

from pyglet import clock

from pyglet.window import key

# Cocos2d
from cocos.director import director
from cocos.tiles import ScrollableLayer
from cocos.layer import Layer
from cocos.scene import Scene
#from cocos.tiles import MapLayer
#from cocos.tiles import RectCell

import logging
log = logging.getLogger('GameLayer')



#
# Local imports
#
from cell import *
from shape import *
from hud import HUD
from level import levelConfig
from gameoverscreen import GameOver

import gamekeys

class GameLayer(Layer):
    is_event_handler = True
    def __init__(self, hud):
        super(GameLayer,self).__init__()
        self.cells = None
        self.nextCell = None
        
        self.gameWidth = 10
        self.gameHeight = 10
        
        self.cellSize = 32
        self.defaultActiveCellPos = [4,0];
        self.activeCellPos = [0,0]
        self.lastCellPos = [0,0]
        self.activeCellPos[0] = self.defaultActiveCellPos[0];
        self.activeCellPos[1] = self.defaultActiveCellPos[1];
        self.activeCell = None#self.GetNextCell();
        
        self.offset_x = 100
        self.offset_y = 100
        
        self.level = None
        self.shoppingList = []
        self.cellChoices = []
        self.HUD = hud
        
        self.SetupBoard()
        
        
        log.info("__init__ complete")
    
    
    def SetupBoard(self):
        
        levelName = levelConfig.get("level","name")
        
        self.HUD.SetLevelInfo(levelName)
        self.HUD.SetSpecialsAvail(eval(levelConfig.get("level", "Specials")))
        
        log.info("Level name: "+levelName)
        
        self.gameWidth = levelConfig.getint("level", "width")
        self.gameHeight = levelConfig.getint("level", "height")
        
        self.cellChoices = eval(levelConfig.get("level", "CellTypes"))
       
        self.nextCell = self.GetNextCell();
        self.add(self.nextCell)
        
        self.cells = Board(self.gameWidth,self.gameHeight)

        self.GetNewCell()
        
        self.level = [] 
        for (shapeDef, data) in levelConfig.items("shapes"):
            needed, special = eval(data)
            shapeObj = MakeShape(string.capwords(shapeDef))
            shapeObj.SetNeeded(needed, special)
            self.level.append(shapeObj)
            
        x = 600
        y = 550
        log.info("level:"+str(self.level))

        for shape in self.level:
            shape.x = x
            shape.y = y
            y -= (shape.boardHeight+1)*32 
            self.add(shape)       
        
        log.info("SetupBoard complete")
   
    def GetNewCell(self):
        log.info("GetNewCell "+str(self.activeCellPos) + " :" +  repr(self.cells.GetColumn([self.activeCellPos[0]])))
        
        self.activeCellPos[0] = self.defaultActiveCellPos[0];
        self.activeCellPos[1] = self.defaultActiveCellPos[1];
        self.lastCellPos = [self.activeCellPos[0], self.activeCellPos[1]]
        
        if ((self.cells.GetCell((self.activeCellPos[0], self.activeCellPos[1])) != None) and (self.cells.GetCell((self.activeCellPos[0],self.activeCellPos[1]+1)) != None)):
            log.info("GetNewCell GAME OVER")
            self.unschedule(self.UpdateCells)
            self.parent.add(GameOver(False, self.HUD.score), z=100)
            return
        
        self.activeCell = self.nextCell
        
        #self.SetCellPos(int(self.activeCellPos[0]), int(self.activeCellPos[1]))
            
        self.nextCell = self.GetNextCell()
        self.nextCell.x = 50
        self.nextCell.y = 500
        
        self.cells.add(self.nextCell)
        self.add(self.nextCell)
         
        #log.info("GetNewCell complete")
        
    def GetNextCell(self):
        log.info("GetNextCell")

        return MakeCell(random.choice(self.cellChoices))
    
    def SetCellPos(self, x, y):
        self.cells.SetCell((self.lastCellPos[0], self.lastCellPos[1]) , None)   
         
        self.activeCell.x = int(self.offset_x+x*self.cellSize)
        self.activeCell.y = 600-int(self.offset_y+y*self.cellSize)
        #log.info("SetCellPos "+str(type(dx))+","+str(type(dy))+" - " +str(type(self.activeCell.cell_ID)))
        #log.info("cells "+repr(self.cells))
        self.cells.SetCell((x,y), self.activeCell)  
        
        self.lastCellPos = (self.activeCellPos[0], self.activeCellPos[1])
        
        #log.info("SetCellPos complete"+str(dx)+","+str(dy))
       
    def UpdateCells(self,dt):

        if gamekeys.keys[key.LEFT]:
            self.OnKeyLeft()
        if gamekeys.keys[key.RIGHT]:
            self.OnKeyRight()
        if gamekeys.keys[key.DOWN]:
            self.OnKeyDown()


        if ((self.activeCellPos[1] < self.gameHeight-1) and (self.cells.GetCell((self.activeCellPos[0],self.activeCellPos[1]+1))== None)):

            self.cells.SetCell((self.activeCellPos[0],self.activeCellPos[1]), None)  
             
            self.activeCellPos[1] += 1
            #log.info("UpdateCells activeCellPos "+repr(self.activeCellPos))
        else:
            #self.SetCellPos(activeCellPos[0], activeCellPos[1])
            
            # Lets do something special
            cellid = self.cells.GetCellID((self.activeCellPos[0],self.activeCellPos[1]))
            if (cellid >= SPECIALS_OFFSET):
                #self.cells.SetCell((self.activeCellPos[0],self.activeCellPos[1]), None)
                # do stuff
                self.ActivateSpecial(cellid-SPECIALS_OFFSET)
                  
            self.CheckCells()
            self.GetNewCell()
            
        self.SetCellPos(self.activeCellPos[0], self.activeCellPos[1])

        #log.info("UpdateCells complete")
     
    def CheckCells(self):
        found = None
        pos = None
        log.info("CheckCells: "+str(self.cells))
        for target in self.level:
            pos = target.CompareShape(self.cells)
            if (pos != None):
                found = target
                break
        
        if (found != None):
            log.info("Waaah found "+found.name)
            
            if (found.UseShape()):
                self.remove(found)               
                self.level.remove(found)
                
            self.HUD.Matched(found)
            # Do we clear now we have matched?
            self.ClearShape( found, pos )
            
            self.CheckForVictory()
      
    def ActivateSpecial(self, specialID):
                
        if (specialID == 0):
            choices = eval(levelConfig.get("level", "Specials"))
            choices.remove(0)
            
            specialID = random.choice(choices)
            log.info("ActivateSpecial randomising...")
            
        log.info("ActivateSpecial "+str(specialID))
        
        # The following is instead of a switch
        {
                  1:self._BombSpecial,
                  2:self._BulldozerSpecial,
                  3:self._FloodSpecial,
                  4:self._TNTSpecial,
                  5:self._TornadoSpecial,  
        }[specialID]()
        
        # Remove the speicial
        self.ClearCell((self.activeCellPos[0],self.activeCellPos[1]))
        

    def _BombSpecial(self):
        log.info("Bomb")
        # Clear everything
        x = 0
        while x < self.cells.boardWidth:
            y = 0
            while y < self.cells.boardHeight:
                self.ClearCell((x,y))
                y += 1
            x += 1
            
    def _BulldozerSpecial(self):
        log.info("Bulldozer @"+str((self.activeCellPos[0],self.activeCellPos[1])))
        offset = random.choice([-1,1])
        self.ClearCell((self.activeCellPos[0] + offset,self.activeCellPos[1]))
        self.cells.ApplyGravity(self.activeCellPos[0] + offset, self.activeCellPos[1])
        self.cells.ApplyGravity(self.activeCellPos[0] , self.activeCellPos[1])
        
    def _FloodSpecial(self):
        log.info("Flood @"+str(self.activeCellPos[1]))
        x = 0
        while x < self.cells.boardWidth:
            self.ClearCell((x,self.activeCellPos[1]))
            self.cells.ApplyGravity(x, self.activeCellPos[1])
            x += 1
            
    def _TNTSpecial(self):
        log.info("TNT @"+str((self.activeCellPos[0],self.activeCellPos[1])))
        self.ClearCell((self.activeCellPos[0] + 1,self.activeCellPos[1]))
        self.ClearCell((self.activeCellPos[0] - 1,self.activeCellPos[1]))
        self.ClearCell((self.activeCellPos[0],self.activeCellPos[1] +1))
        self.ClearCell((self.activeCellPos[0],self.activeCellPos[1] -1))
        
        self.cells.ApplyGravity(self.activeCellPos[0] + 1, self.activeCellPos[1])
        self.cells.ApplyGravity(self.activeCellPos[0] , self.activeCellPos[1])
        self.cells.ApplyGravity(self.activeCellPos[0] - 1, self.activeCellPos[1])

    def _TornadoSpecial(self):
        log.info("Tornado @"+str(self.activeCellPos[0]))
        y = 0
        while y < self.cells.boardHeight:
            self.ClearCell((self.activeCellPos[0],y))
            y += 1

    
    def CreateSpecial(self, activateIndex):    
        log.info("Player tried to acivate special slot #"+str(activateIndex))   
             
        active, cellID = self.HUD.IsSpecialActive(activateIndex)
        if (active):
            self.HUD.SetSpecialActive(activateIndex, False)
            self.remove(self.nextCell)
            self.nextCell = MakeCell(cellID)
            self.add(self.nextCell)
            
            
    def ClearShape(self, shape, pos):  
        '''Remove other cells from this shape'''
        sx,sy = pos
        #log.info("ClearShape: "+str(shape)+" at "+str((sx,sy))+" shape: "+str((shape.boardWidth,shape.boardHeight)))
        
        while sx-pos[0] < shape.boardWidth:
            sy = pos[1]
            while sy-pos[1] < shape.boardHeight:
                self.ClearCell((sx,sy))
                sy += 1
            self.cells.ApplyGravity(sx, sy-1)
            sx += 1
            
    def ClearCell(self, pos):   
        if (pos[0] <0 or pos[1] <0 or pos[0] >= self.cells.boardWidth or pos[1] >= self.cells.boardHeight):
            return
             
        cell = self.cells.GetCell(pos)
        if (cell != None):
            self.remove(cell)
            # Clear cell
            self.cells.SetCell(pos, None)

    def CheckForVictory(self):
        # what are the victory conditions?
        # 1. finish shopping list
        # 2. board clear
        needed = 0
        for shape in self.level:
            needed += shape.numberNeeded

        if (needed == 0):
            # Win on isle 1!
            self.unschedule(self.UpdateCells)
            self.parent.add(GameOver(True, self.HUD.score), z=100)

    def StartGame(self):
        self.schedule_interval(self.UpdateCells,1.0/5.0)

    def OnKeyRight(self):
        if ((self.activeCellPos[0]+1 <self.gameWidth-1) and self.cells.GetCell((self.activeCellPos[0]+1,self.activeCellPos[1])) == None):
            self.activeCellPos[0] +=1
            self.SetCellPos(self.activeCellPos[0], self.activeCellPos[1])
            
    def OnKeyLeft(self):        
        if ((self.activeCellPos[0]-1 >=0) and self.cells.GetCell((self.activeCellPos[0]-1,self.activeCellPos[1])) == None):
            self.activeCellPos[0] -=1
            self.SetCellPos(self.activeCellPos[0], self.activeCellPos[1])
            
    def OnKeyDown(self):        
        if ((self.activeCellPos[1] < self.cells.boardHeight -1) and self.cells.GetCell((self.activeCellPos[0],self.activeCellPos[1]+1)) == None):
            self.activeCellPos[1] +=1
            self.SetCellPos(self.activeCellPos[0], self.activeCellPos[1])
            
    def on_key_press (self, keys, modifiers):
        
        # Prehaps we can rely on held keys only? hmm not sure prob not   
        if (keys == key.RIGHT ):
            self.OnKeyRight()
            return
        
        if (keys == key.LEFT ):
            self.OnKeyLeft()
            return
        
        if (gamekeys.keys == key.DOWN ):
            self.OnKeyDown()
            return
        
        if (keys == key.R ):
            log.info("Restart requested.")
            #children = self.get_children
            self.SetupBoard()
            self.StartGame()
            return
        
        if (keys == key.U ):
            self.UpdateCells()
            return
          
            
        if (keys == key.F1):
            log.info("Leave game.")
            self.unschedule(self.UpdateCells)
            self.parent.parent.switch_to(0)
        
        if (keys == key._1 ):
            self.CreateSpecial(0)
            return
        if (keys == key._2 ):
            self.CreateSpecial(1)
            return
        if (keys == key._3 ):
            self.CreateSpecial(2)
            return
        if (keys == key._4 ):
            self.CreateSpecial(3)
            return
        if (keys == key._5 ):
            self.CreateSpecial(4)
            return
        if (keys == key._6 ):
            self.CreateSpecial(5)
            return
            

    def on_key_release (self, key, modifiers):
        pass
        #for player in gPlayers:
        #    player.KeyRelease(key)

    def on_enter(self):
        super( GameLayer, self).on_enter()
        self.is_event_handler = True
        self.StartGame()
        
    def on_exit(self):
        super( GameLayer, self).on_exit()
        self.is_event_handler = False        
        self.unschedule(self.UpdateCells)
    
