#
# Python includes
#
import os, sys
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

import logging
import math

#
# Library includes
#

# Pygame
import pygame
from pygame.locals import *

# Cocos2d
from cocos.director import director
from cocos.scene import Scene
from cocos.layer import Layer
from cocos.sprite import Sprite



import pyglet
from pyglet import font   
from pyglet.window import key



path = os.path.abspath(os.path.dirname(__file__)+"/../data") # odd - windows and cmd line dont like the '/'

pyglet.resource.path.append(path)
pyglet.resource.path.append(path+"/art")
pyglet.resource.path.append(path+"/specials")
pyglet.resource.path.append(path+"/menu")
pyglet.resource.reindex()

font.add_directory(path+"/gamefonts")
#arial = font.load('m02.TTF', 14, bold=True, italic=False)

#
# Local imports
#
from gamescene import *
from hud import HUD


from mainmenu import MainMenu

import gamekeys

#
# Setup logging
#
logging.basicConfig(level = logging.DEBUG,
                    format   ='%(asctime)s %(name)-12s %(levelname)-8s %(message)s',
                    datefmt  ='%H:%M:%S',
                    filename = 'test.log',
                    filemode = 'w')
                    
# define a Handler which writes INFO messages or higher to the sys.stderr
console = logging.StreamHandler()
console.setLevel(logging.DEBUG)
# set a format which is simpler for console use
formatter = logging.Formatter('%(name)-12s: %(levelname)-8s %(message)s')
# tell the handler to use this format
console.setFormatter(formatter)
# add the handler to the root logger
logging.getLogger('').addHandler(console)

log = logging.getLogger('main')
log.info('Starting main...')

class ImageLayer(Layer):
    def __init__(self, name):
        super( ImageLayer, self ).__init__()
        
        x,y = director.get_window_size()

        self.sprite = Sprite( name, (x*0.5,y*0.7) )
        self.add( self.sprite )
        


# Main program
def run():
    
    director.init(width=800, height=600, do_not_scale=True)

    director.window.set_caption("NaN - Pyweek8 - Suburbia")

    gamekeys.keys = key.KeyStateHandler()
    director.window.push_handlers(gamekeys.keys)

    
    scene = Scene(MainMenu(), ImageLayer('suburbialogo.png'))
    
    director.run (scene)
        
    log.info("END")
    
if __name__ == "__main__":
    run()
